@@ -205,3 +205,18 @@ def test_write_role_and_group(datadir, tmpdir):
     gold_file = datadir / "entity_with_role_and_group.conllx"
 
     assert filecmp.cmp(result_file, gold_file)
+
+def test_apply(datadir):
+    from tokenizer_tools.tagset.offset.transform import text_to_lower
+
+    corpus = Corpus()
+
+    doc = Document("打开 Bilibili APP", id="1")
+    corpus.append(doc)
+    doc = Document("iPhone 手机", id="2")
+    corpus.append(doc)
+
+    corpus.apply(text_to_lower)
+
+    assert len(corpus) == 2
+    assert all(map(lambda x: x.lower() == x, corpus[0].text))
