@@ -1,11 +1,13 @@
 import itertools
 from typing import Dict, List
+import pathlib
 import collections
 
 from tokenizer_tools.tagset.offset.analysis.express_pattern import \
     ExpressPattern
 from tokenizer_tools.tagset.offset.corpus import Corpus
 from tokenizer_tools.tagset.offset.analysis.read_markdown import read_markdown
+from tokenizer_tools.tagset.offset.analysis.plain_text import parse_plain_text
 
 
 # TODO: PatternCorpus seems is a better name than this
@@ -52,11 +54,28 @@ class CorpusPattern(Corpus):
 
     @classmethod
     def read_from_file(cls, data_file):
+        file_ext = pathlib.Path(data_file).suffix
+        if file_ext == ".md":
+            return cls._read_from_md(data_file)
+        elif file_ext == ".txt":
+            return cls._read_from_txt(data_file)
+        else:
+            raise ValueError("file type not supported, currently only support .md and .txt")
+
+    @classmethod
+    def _read_from_md(cls, data_file):
         with open(data_file) as fd:
             doc_list = read_markdown(fd.read())
 
         return cls(doc_list)
 
+    @classmethod
+    def _read_from_txt(cls, data_file):
+        with open(data_file) as fd:
+            self = parse_plain_text(fd.read())
+
+        return self
+
     def convert_to_md(self) -> str:
         output = ""
 
