@@ -1,4 +1,5 @@
 from typing import Union, List, Tuple, Callable
+import pathlib
 
 import numpy as np
 from sklearn.model_selection import train_test_split
@@ -34,6 +35,22 @@ class Corpus(List[Document]):
 
     @classmethod
     def read_from_file(cls, data_file):
+        file_ext = pathlib.Path(data_file).suffix
+        if file_ext == ".conllx":
+            return cls._read_from_conllx_file(data_file)
+        elif file_ext == ".pb":
+            return cls._read_from_pb_file(data_file)
+        else:
+            raise ValueError("file type not supported, currently only support .conllx and .pb")
+
+    @classmethod
+    def _read_from_pb_file(cls, data_file) -> "Corpus":
+        from tokenizer_tools.tagset.offset.corpus_protobuf import load_corpus_from_pb
+
+        return load_corpus_from_pb(data_file)
+
+    @classmethod
+    def _read_from_conllx_file(cls, data_file) -> "Corpus":
         offset_data_list = []
 
         with open(data_file, "rt") as fd:
@@ -42,7 +59,10 @@ class Corpus(List[Document]):
 
                 offset_data_list.append(offset_data)
 
-        return Corpus(offset_data_list)
+        self = Corpus(offset_data_list)
+        self._build_id_to_doc_mapping()
+        self._build_txt_to_doc_mapping()
+        return self
 
     @classmethod
     def read_from_dir(cls, data_dir) -> "Corpus":
@@ -50,6 +70,17 @@ class Corpus(List[Document]):
         """
         raise NotImplementedError()
 
+    def __init__(self, iterable=()):
+        super().__init__(iterable)
+        self._txt_to_doc = None
+        self._id_to_doc = None
+
+    def _build_txt_to_doc_mapping(self):
+        self._txt_to_doc = {doc.convert_to_md(): doc for doc in self}
+
+    def _build_id_to_doc_mapping(self):
+        self._id_to_doc = {doc.id: doc for doc in self}
+
     def write_to_file(self, output_file):
         sentence_list = [offset_to_sentence(offset) for offset in self]
 
@@ -162,10 +193,13 @@ class Corpus(List[Document]):
         return id_list
 
     def get_doc_by_id(self, doc_id, default=None):
-        for doc in self:
-            if doc.id == doc_id:
-                return doc
-        return default
+        if self._id_to_doc is None:
+            for doc in self:
+                if doc.id == doc_id:
+                    return doc
+            return default
+        else:
+            return self._id_to_doc.get(doc_id, default)
 
     def fuzzy_search(self, query, limit=3) -> List[Tuple[Document, int]]:
         choices = self
@@ -219,3 +253,11 @@ class Corpus(List[Document]):
         """merge a list of corpora into one corpus
         """
         raise NotImplementedError()
+
+    def apply(self, func) -> "Corpus":
+        """Apply function *inplace* to corpus
+        """
+        # map is lazy evaluated, using list() to force execute now
+        list(map(func, self))
+
+        return self
